"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ShellCommand = void 0;
const helper_1 = require("../helper");
const pattern_parser_1 = require("../pattern.parser");
const shell_registry_1 = require("../shell.registry");
const value_to_param_mapper_1 = require("../value-to-param.mapper");
function ShellCommand(options) {
    return (target, methodName, descriptor) => {
        const { name, prefix = '', description = '', pattern = '' } = options;
        const handler = async (input, messages, print) => {
            const componentInstance = shell_registry_1.ShellRegistry.getComponent(target.constructor.name);
            const commandMethod = componentInstance[methodName];
            const patternParams = _getParsedPatternParams(pattern, commandMethod, input);
            if (_hasAnyRequiredParam(patternParams)) {
                const commandName = (prefix || '') + name;
                print(messages.wrongUsage.replace('$command', commandName).replace('$pattern', pattern));
                return;
            }
            return commandMethod
                .apply(componentInstance, _mapParamsToValueOnly(patternParams))
                .then((result) => print(result));
        };
        shell_registry_1.ShellRegistry.registerCommand({ name, prefix, description, pattern, handler });
    };
}
exports.ShellCommand = ShellCommand;
function _getParsedPatternParams(pattern, commandMethod, input) {
    let patternParams = pattern_parser_1.parsePattern(pattern);
    patternParams = _replaceSignatureIndex(patternParams, helper_1.getFunctionArgs(commandMethod));
    patternParams = value_to_param_mapper_1.mapActualValueToParams(patternParams, input);
    return patternParams;
}
function _replaceSignatureIndex(patternParams, functionArguments) {
    let mutablePatternParams = helper_1.deepClone(patternParams);
    _ensureAnyPatternParamCanBeBinded(patternParams, functionArguments);
    functionArguments.forEach((arg, index) => {
        mutablePatternParams = Object.assign(Object.assign({}, mutablePatternParams), { [arg]: Object.assign(Object.assign({}, mutablePatternParams[arg]), { signatureIndex: index }) });
    });
    return mutablePatternParams;
}
function _ensureAnyPatternParamCanBeBinded(patternParams, functionArguments) {
    const patternParamsCount = Object.values(patternParams).length;
    if (patternParamsCount > 0 && functionArguments < patternParamsCount) {
        const lastParamName = Object.keys(patternParams).slice(-1);
        throw Error(`Parameter specified in pattern has no equaivalent in actual parameters [Param name: ${lastParamName}]`);
    }
}
function _hasAnyRequiredParam(patternParams) {
    const requiredParamsWithNoValue = patternParams.filter((param) => param.isRequired && !param.value);
    return requiredParamsWithNoValue.length > 0;
}
function _mapParamsToValueOnly(patternParams) {
    return patternParams.map((param) => param.value);
}
//# sourceMappingURL=shell-command.decorator.js.map